"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("./constants");
function main() {
    if (!process.connected) {
        console.error('IPC channel not exists.');
        process.exit(1);
    }
    console.info('Lambda process started.');
    /**
     * Initialize a current process.
     */
    process.on('exit', code => {
        console.info('Lambda process exits with code %d', code);
    });
    process.on('message', function (message, sendHandle) {
        this.emit(message.type, message);
    });
    process.on(constants_1.INIT, initHandler);
    process.on(constants_1.INVOKE, invokeHandler);
}
function initHandler(message) {
    const { arn, module, handlerName, } = message;
    if (arn) {
        process.title = arn;
    }
    let lambdaModule;
    try {
        lambdaModule = require(module);
    }
    catch (err) {
        sendLastMessage(this, setError({ type: constants_1.INIT_RESULT }, err));
        return;
    }
    const lambdaHandler = lambdaModule[handlerName];
    if (typeof lambdaHandler !== 'function') {
        const err = new Error(`Module '${module}' does not export the handler '${handlerName}'.`);
        sendLastMessage(this, setError({ type: constants_1.INIT_RESULT }, err));
        return;
    }
    this.lambdaHandler = lambdaHandler;
    this.send({
        type: constants_1.INIT_RESULT,
        err: null,
    });
}
function invokeHandler(message) {
    const { event, context, } = message;
    if (this.lambdaHandler.length === 3) {
        this.lambdaHandler(event, context, (err, result) => this.send(setError({
            type: constants_1.INVOKE_RESULT,
            msgId: message.msgId,
            result,
        }, err), (err) => {
            if (err) {
                this.disconnect();
            }
        }));
    }
    else {
        this.lambdaHandler(event, context)
            .then(result => {
            this.send({
                type: constants_1.INVOKE_RESULT,
                msgId: message.msgId,
                result,
            }, (err) => {
                if (err) {
                    this.disconnect();
                }
            });
        })
            .catch(err => {
            this.send(setError({
                type: constants_1.INVOKE_RESULT,
                msgId: message.msgId,
            }, err), (err) => {
                if (err) {
                    this.disconnect();
                }
            });
        });
    }
}
/**
 * Send a message and disconnect regardless of success or failure.
 */
function sendLastMessage(process, message) {
    process.send(message, (err) => process.disconnect());
}
function setError(obj, err) {
    if (err) {
        obj.err = {
            code: err.code,
            message: err.message,
            stack: err.stack,
        };
    }
    return obj;
}
main();
