"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.spawnLambda = void 0;
const child_process_1 = require("child_process");
const path_1 = require("path");
const constants_1 = require("./constants");
const makeOnceCallback_1 = require("./makeOnceCallback");
const debug = require('debug')('lambda-spawn:spawnLambda');
function spawnLambda(options) {
    const { typescript, } = options;
    const { arn, dir = process.cwd(), handler = 'index.handler', command = typescript ? 'ts-node' : 'node', args = [constants_1.DEFAULT_DRIVER_PATH], env, lambdaEnv, additionalNodePath, stdio = ['ignore', 'inherit', 'inherit'], project, } = options;
    const { moduleDir = dir, } = options;
    const [, stdout = 'inherit', stderr = 'inherit'] = stdio;
    if (project) {
        args.unshift('-P', project);
    }
    const mergedEnv = Object.assign({}, env || process.env, lambdaEnv);
    const { LAMBDA_TASK_ROOT = mergedEnv.LAMBDA_TASK_ROOT = moduleDir, LAMBDA_RUNTIME_DIR = mergedEnv.LAMBDA_RUNTIME_DIR = moduleDir, } = mergedEnv;
    mergedEnv.NODE_PATH = [
        mergedEnv.NODE_PATH,
        LAMBDA_RUNTIME_DIR,
        LAMBDA_TASK_ROOT,
        LAMBDA_RUNTIME_DIR + '/node_modules',
        additionalNodePath,
    ].filter(s => s).join(':');
    const spawnOptions = {
        cwd: dir,
        env: mergedEnv,
        stdio: ['ignore', stdout, stderr, 'ipc'],
    };
    const lambdaProcess = spawnProcess(command, args, spawnOptions);
    lambdaProcess.arn = arn;
    const [moduleBasename, handlerName] = handler.split('.');
    lambdaProcess.send({
        type: constants_1.INIT,
        arn,
        module: (0, path_1.join)((0, path_1.resolve)(moduleDir), moduleBasename + (typescript ? '.ts' : '.js')),
        handlerName,
    });
    return lambdaProcess;
}
exports.spawnLambda = spawnLambda;
function spawnProcess(command, args, options) {
    const process = (0, child_process_1.spawn)(command, args, options);
    process._msgId = 0;
    process.on('close', (code, signal) => {
        debug('Lambda process closed. code = %d, signal = %s', code, signal);
    });
    process.on('disconnect', () => {
        debug('Lambda process disconnected');
    });
    process.on('error', err => {
        debug('Lambda process error. err = %j', err);
    });
    process.on('exit', (code, signal) => {
        debug('Lambda process exited. code = %d, signal = %s', code, signal);
    });
    process.on('message', function (message, sendHandle) {
        debug('Lambda process returns a result %j', message);
        this.emit(message.type, message);
    });
    process.invoke = function (event, context, callback) {
        callback = (0, makeOnceCallback_1.makeOnceCallback)(callback);
        const invokeContext = {};
        invokeContext.msgId = ++this._msgId;
        invokeContext.resultHandler = (message) => {
            if (message.msgId !== invokeContext.msgId) {
                return;
            }
            callback(message.err, message.result);
            invokeContext.destroy();
        };
        invokeContext.exitHandler = (code, signal) => {
            callback(new Error(`Lambda process exited. code = ${code}, signal = ${signal}`));
            invokeContext.destroy();
        };
        invokeContext.destroy = () => {
            this.removeListener(constants_1.INVOKE_RESULT, invokeContext.resultHandler);
            this.removeListener('exit', invokeContext.exitHandler);
            delete invokeContext.resultHandler;
            delete invokeContext.exitHandler;
            delete invokeContext.destroy;
        };
        this.on(constants_1.INVOKE_RESULT, invokeContext.resultHandler);
        this.on('exit', invokeContext.exitHandler);
        const invokeMessage = {
            type: constants_1.INVOKE,
            event,
            context,
            msgId: invokeContext.msgId,
        };
        this.send(invokeMessage, err => {
            if (err) {
                callback(err);
                invokeContext.destroy();
            }
        });
    };
    return process;
}
